@@ -1,25 +1,28 @@
-#
-#                _         _    _      _                _    
-#               (_)       | |  | |    | |              | |   
-#  _ __    ___   _  _ __  | |_ | |__  | |  __ _  _ __  | | __
-# | '_ \  / _ \ | || '_ \ | __|| '_ \ | | / _` || '_ \ | |/ /
-# | |_) || (_) || || | | || |_ | |_) || || (_| || | | ||   < 
-# | .__/  \___/ |_||_| |_| \__||_.__/ |_| \__,_||_| |_||_|\_\
-# | |                                                        
-# |_|                                                        
+#------------------------------------------------------------------------------#
 # 
-# This file is part of the 'rich-iannone/pointblank' package.
+#                 _         _    _      _                _    
+#                (_)       | |  | |    | |              | |   
+#   _ __    ___   _  _ __  | |_ | |__  | |  __ _  _ __  | | __
+#  | '_ \  / _ \ | || '_ \ | __|| '_ \ | | / _` || '_ \ | |/ /
+#  | |_) || (_) || || | | || |_ | |_) || || (_| || | | ||   < 
+#  | .__/  \___/ |_||_| |_| \__||_.__/ |_| \__,_||_| |_||_|\_\
+#  | |                                                        
+#  |_|                                                        
+#  
+#  This file is part of the 'rstudio/pointblank' project.
+#  
+#  Copyright (c) 2017-2024 pointblank authors
+#  
+#  For full copyright and license information, please look at
+#  https://rstudio.github.io/pointblank/LICENSE.html
 # 
-# (c) Richard Iannone <riannone@me.com>
-# 
-# For full copyright and license information, please look at
-# https://rich-iannone.github.io/pointblank/LICENSE.html
-#
+#------------------------------------------------------------------------------#
 
 
 #' Do one or more columns actually exist?
 #'
 #' @description
+#' 
 #' The `col_exists()` validation function, the `expect_col_exists()` expectation
 #' function, and the `test_col_exists()` test function all check whether one or
 #' more columns exist in the target table. The only requirement is specification
@@ -29,7 +32,17 @@
 #' table. Each validation step or expectation will operate over a single test
 #' unit, which is whether the column exists or not.
 #' 
+#' @inheritParams col_vals_gt
+#' 
+#' @return For the validation function, the return value is either a
+#'   `ptblank_agent` object or a table object (depending on whether an agent
+#'   object or a table was passed to `x`). The expectation function invisibly
+#'   returns its input but, in the context of testing data, the function is
+#'   called primarily for its potential side-effects (e.g., signaling failure).
+#'   The test function returns a logical value.
+#' 
 #' @section Supported Input Tables:
+#' 
 #' The types of data tables that are officially supported are:
 #' 
 #'  - data frames (`data.frame`) and tibbles (`tbl_df`)
@@ -47,14 +60,21 @@
 #' **pointblank**).
 #'
 #' @section Column Names:
-#' If providing multiple column names, the result will be an expansion of
-#' validation steps to that number of column names (e.g., `vars(col_a, col_b)`
-#' will result in the entry of two validation steps). Aside from column names in
-#' quotes and in `vars()`, **tidyselect** helper functions are available for
-#' specifying columns. They are: `starts_with()`, `ends_with()`, `contains()`,
-#' `matches()`, and `everything()`.
+#' 
+#' `columns` may be a single column (as symbol `a` or string `"a"`) or a vector
+#' of columns (`c(a, b, c)` or `c("a", "b", "c")`). `{tidyselect}` helpers
+#' are also supported, such as `contains("date")` and `where(is.double)`. If
+#' passing an *external vector* of columns, it should be wrapped in `all_of()`.
+#' 
+#' When multiple columns are selected by `columns`, the result will be an
+#' expansion of validation steps to that number of columns (e.g.,
+#' `c(col_a, col_b)` will result in the entry of two validation steps).
+#' 
+#' Previously, columns could be specified in `vars()`. This continues to work, 
+#' but `c()` offers the same capability and supersedes `vars()` in `columns`.
 #'
 #' @section Actions:
+#' 
 #' Often, we will want to specify `actions` for the validation. This argument,
 #' present in every validation function, takes a specially-crafted list object
 #' that is best produced by the [action_levels()] function. Read that function's
@@ -66,7 +86,20 @@
 #' depending on the situation (the first produces a warning, the other
 #' `stop()`s).
 #'
+#' @section Labels:
+#' 
+#' `label` may be a single string or a character vector that matches the number
+#' of expanded steps. `label` also supports `{glue}` syntax and exposes the
+#' following dynamic variables contextualized to the current step:
+#'   
+#' - `"{.step}"`: The validation step name
+#' - `"{.col}"`: The current column name
+#'     
+#' The glue context also supports ordinary expressions for further flexibility
+#' (e.g., `"{toupper(.step)}"`) as long as they return a length-1 string.
+#' 
 #' @section Briefs:
+#' 
 #' Want to describe this validation step in some detail? Keep in mind that this
 #' is only useful if `x` is an *agent*. If that's the case, `brief` the agent
 #' with some text that fits. Don't worry if you don't want to do it. The
@@ -74,6 +107,7 @@
 #' then be automatically generated.
 #' 
 #' @section YAML:
+#' 
 #' A **pointblank** agent can be written to YAML with [yaml_write()] and the
 #' resulting YAML can be used to regenerate an agent (with [yaml_read_agent()])
 #' or interrogate the target table (via [yaml_agent_interrogate()]). When
@@ -88,7 +122,7 @@
 #' ```r
 #' agent %>% 
 #'   col_exists(
-#'     columns = vars(a),
+#'     columns = a,
 #'     actions = action_levels(warn_at = 0.1, stop_at = 0.2),
 #'     label = "The `col_exists()` step.",
 #'     active = FALSE
@@ -100,7 +134,7 @@
 #' ```yaml
 #' steps:
 #' - col_exists:
-#'     columns: vars(a)
+#'     columns: c(a)
 #'     actions:
 #'       warn_fraction: 0.1
 #'       stop_fraction: 0.2
@@ -114,18 +148,6 @@
 #' their default when generating the YAML by other means). It is also possible
 #' to preview the transformation of an agent to YAML without any writing to disk
 #' by using the [yaml_agent_string()] function.
-#'
-#' @inheritParams col_vals_gt
-#' @param columns One or more columns from the table in focus. This can be
-#'   provided as a vector of column names using `c()` or bare column names
-#'   enclosed in [vars()].
-#'   
-#' @return For the validation function, the return value is either a
-#'   `ptblank_agent` object or a table object (depending on whether an agent
-#'   object or a table was passed to `x`). The expectation function invisibly
-#'   returns its input but, in the context of testing data, the function is
-#'   called primarily for its potential side-effects (e.g., signaling failure).
-#'   The test function returns a logical value.
 #'   
 #' @section Examples:
 #' 
@@ -151,7 +173,7 @@
 #' ```r
 #' agent <-
 #'   create_agent(tbl = tbl) %>%
-#'   col_exists(columns = vars(a)) %>%
+#'   col_exists(columns = a) %>%
 #'   interrogate()
 #' ```
 #' 
@@ -172,7 +194,7 @@
 #' The behavior of side effects can be customized with the `actions` option.
 #' 
 #' ```{r}
-#' tbl %>% col_exists(columns = vars(a))
+#' tbl %>% col_exists(columns = a)
 #' ```
 #' 
 #' ## C: Using the expectation function
@@ -181,7 +203,7 @@
 #' time. This is primarily used in **testthat** tests.
 #' 
 #' ```r
-#' expect_col_exists(tbl, columns = vars(a))
+#' expect_col_exists(tbl, columns = a)
 #' ```
 #' 
 #' ## D: Using the test function
@@ -190,7 +212,7 @@
 #' us.
 #' 
 #' ```{r}
-#' tbl %>% test_col_exists(columns = vars(a))
+#' tbl %>% test_col_exists(columns = a)
 #' ```
 #' 
 #' @family validation functions
@@ -205,7 +227,7 @@ NULL
 #' @export
 col_exists <- function(
     x,
-    columns,
+    columns = NULL,
     actions = NULL,
     step_id = NULL,
     label = NULL,
@@ -216,21 +238,26 @@ col_exists <- function(
   preconditions <- NULL
   values <- NULL
   
+  # Capture the `columns` expression
+  columns <- rlang::enquo(columns)
   # Get `columns` as a label
-  columns_expr <- 
-    rlang::as_label(rlang::quo(!!enquo(columns))) %>%
-    gsub("^\"|\"$", "", .)
+  columns_expr <- as_columns_expr(columns)
   
-  # Normalize the `columns` expression
-  if (inherits(columns, "quosures")) {
-    
-    columns <- 
-      vapply(
-        columns,
-        FUN.VALUE = character(1),
-        USE.NAMES = FALSE,
-        FUN = function(x) as.character(rlang::get_expr(x))
-      )
+  # Resolve the columns based on the expression
+  ## Only for `col_exists()`: error gracefully if column not found
+  columns <- tryCatch(
+    expr = resolve_columns(x = x, var_expr = columns, preconditions = NULL,
+                           allow_empty = FALSE),
+    error = function(cnd) cnd$i %||% cnd
+  )
+  ## Missing column selection
+  if (rlang::is_error(columns)) {
+    cnd <- columns
+    if (inherits(cnd, "resolve_eval_err")) {
+      # Evaluation errors should be rethrown
+      rlang::cnd_signal(cnd)
+    }
+    columns <- NA_character_
   }
 
   if (is_a_table_object(x)) {
@@ -238,7 +265,7 @@ col_exists <- function(
     secret_agent <- 
       create_agent(x, label = "::QUIET::") %>%
       col_exists(
-        columns = columns,
+        columns = tidyselect::all_of(columns),
         actions = prime_actions(actions),
         label = label,
         brief = brief,
@@ -268,7 +295,8 @@ col_exists <- function(
   
   # Add one or more validation steps based on the
   # length of the `columns` variable
-  for (i in seq(columns)) {
+  label <- resolve_label(label, columns)
+  for (i in seq_along(columns)) {
     
     agent <-
       create_validation_step(
@@ -280,7 +308,7 @@ col_exists <- function(
         preconditions = NULL,
         actions = covert_actions(actions, agent),
         step_id = step_id[i],
-        label = label,
+        label = label[[i]],
         brief = brief[i],
         active = active
       )
