@@ -1,25 +1,28 @@
-#
-#                _         _    _      _                _    
-#               (_)       | |  | |    | |              | |   
-#  _ __    ___   _  _ __  | |_ | |__  | |  __ _  _ __  | | __
-# | '_ \  / _ \ | || '_ \ | __|| '_ \ | | / _` || '_ \ | |/ /
-# | |_) || (_) || || | | || |_ | |_) || || (_| || | | ||   < 
-# | .__/  \___/ |_||_| |_| \__||_.__/ |_| \__,_||_| |_||_|\_\
-# | |                                                        
-# |_|                                                        
+#------------------------------------------------------------------------------#
 # 
-# This file is part of the 'rich-iannone/pointblank' package.
+#                 _         _    _      _                _    
+#                (_)       | |  | |    | |              | |   
+#   _ __    ___   _  _ __  | |_ | |__  | |  __ _  _ __  | | __
+#  | '_ \  / _ \ | || '_ \ | __|| '_ \ | | / _` || '_ \ | |/ /
+#  | |_) || (_) || || | | || |_ | |_) || || (_| || | | ||   < 
+#  | .__/  \___/ |_||_| |_| \__||_.__/ |_| \__,_||_| |_||_|\_\
+#  | |                                                        
+#  |_|                                                        
+#  
+#  This file is part of the 'rstudio/pointblank' project.
+#  
+#  Copyright (c) 2017-2024 pointblank authors
+#  
+#  For full copyright and license information, please look at
+#  https://rstudio.github.io/pointblank/LICENSE.html
 # 
-# (c) Richard Iannone <riannone@me.com>
-# 
-# For full copyright and license information, please look at
-# https://rich-iannone.github.io/pointblank/LICENSE.html
-#
+#------------------------------------------------------------------------------#
 
 
 #' Are column data decreasing by row?
 #'
 #' @description
+#' 
 #' The `col_vals_decreasing()` validation function, the
 #' `expect_col_vals_decreasing()` expectation function, and the
 #' `test_col_vals_decreasing()` test function all check whether column values in
@@ -33,7 +36,37 @@
 #' over the number of test units that is equal to the number of rows in the
 #' table (after any `preconditions` have been applied).
 #' 
+#' @inheritParams col_vals_gt
+#' 
+#' @param allow_stationary *Allowance for stationary pauses in values*
+#' 
+#'   `scalar<logical>` // *default:* `FALSE`
+#' 
+#'   An option to allow pauses in decreasing values. For example if the values
+#'   for the test units are `[85, 82, 82, 80, 77]` then the third unit (`82`,
+#'   appearing a second time) would be marked with *fail* when
+#'   `allow_stationary` is `FALSE`. Using `allow_stationary = TRUE` will result
+#'   in all the test units in `[85, 82, 82, 80, 77]` to be marked with *pass*.
+#'   
+#' @param increasing_tol *Optional tolerance threshold for backtracking*
+#' 
+#'   `scalar<numeric>(val>=0)` // *default:* `NULL` (`optional`)
+#' 
+#'   An optional threshold value that allows for movement of numerical values in
+#'   the positive direction. By default this is `NULL` but using a numerical
+#'   value with set the absolute threshold of positive travel allowed across
+#'   numerical test units. Note that setting a value here also has the effect of
+#'   setting `allow_stationary` to `TRUE`.
+#' 
+#' @return For the validation function, the return value is either a
+#'   `ptblank_agent` object or a table object (depending on whether an agent
+#'   object or a table was passed to `x`). The expectation function invisibly
+#'   returns its input but, in the context of testing data, the function is
+#'   called primarily for its potential side-effects (e.g., signaling failure).
+#'   The test function returns a logical value.
+#' 
 #' @section Supported Input Tables:
+#' 
 #' The types of data tables that are officially supported are:
 #' 
 #'  - data frames (`data.frame`) and tibbles (`tbl_df`)
@@ -51,20 +84,28 @@
 #' **pointblank**).
 #'
 #' @section Column Names:
-#' If providing multiple column names to `columns`, the result will be an
-#' expansion of validation steps to that number of column names (e.g.,
-#' `vars(col_a, col_b)` will result in the entry of two validation steps). Aside
-#' from column names in quotes and in `vars()`, **tidyselect** helper functions
-#' are available for specifying columns. They are: `starts_with()`,
-#' `ends_with()`, `contains()`, `matches()`, and `everything()`.
+#' 
+#' `columns` may be a single column (as symbol `a` or string `"a"`) or a vector
+#' of columns (`c(a, b, c)` or `c("a", "b", "c")`). `{tidyselect}` helpers
+#' are also supported, such as `contains("date")` and `where(is.double)`. If
+#' passing an *external vector* of columns, it should be wrapped in `all_of()`.
+#' 
+#' When multiple columns are selected by `columns`, the result will be an
+#' expansion of validation steps to that number of columns (e.g.,
+#' `c(col_a, col_b)` will result in the entry of two validation steps).
+#' 
+#' Previously, columns could be specified in `vars()`. This continues to work, 
+#' but `c()` offers the same capability and supersedes `vars()` in `columns`.
 #'
 #' @section Missing Values:
+#' 
 #' This validation function supports special handling of `NA` values. The
 #' `na_pass` argument will determine whether an `NA` value appearing in a test
 #' unit should be counted as a *pass* or a *fail*. The default of `na_pass =
 #' FALSE` means that any `NA`s encountered will accumulate failing test units.
 #' 
 #' @section Preconditions:
+#' 
 #' Providing expressions as `preconditions` means **pointblank** will preprocess
 #' the target table during interrogation as a preparatory step. It might happen
 #' that a particular validation requires a calculated column, some filtering of
@@ -83,6 +124,7 @@
 #' be supplied (e.g., `function(x) dplyr::mutate(x, col_b = col_a + 10)`).
 #' 
 #' @section Segments:
+#' 
 #' By using the `segments` argument, it's possible to define a particular
 #' validation with segments (or row slices) of the target table. An optional
 #' expression or set of expressions that serve to segment the target table by
@@ -113,6 +155,7 @@
 #' generate a separate version of the target table.
 #' 
 #' @section Actions:
+#' 
 #' Often, we will want to specify `actions` for the validation. This argument,
 #' present in every validation function, takes a specially-crafted list
 #' object that is best produced by the [action_levels()] function. Read that
@@ -127,7 +170,22 @@
 #' quarter of the total test units fails, the other `stop()`s at the same
 #' threshold level).
 #' 
+#' @section Labels:
+#' 
+#' `label` may be a single string or a character vector that matches the number
+#' of expanded steps. `label` also supports `{glue}` syntax and exposes the
+#' following dynamic variables contextualized to the current step:
+#'   
+#' - `"{.step}"`: The validation step name
+#' - `"{.col}"`: The current column name
+#' - `"{.seg_col}"`: The current segment's column name
+#' - `"{.seg_val}"`: The current segment's value/group
+#'     
+#' The glue context also supports ordinary expressions for further flexibility
+#' (e.g., `"{toupper(.step)}"`) as long as they return a length-1 string.
+#' 
 #' @section Briefs:
+#' 
 #' Want to describe this validation step in some detail? Keep in mind that this
 #' is only useful if `x` is an *agent*. If that's the case, `brief` the agent
 #' with some text that fits. Don't worry if you don't want to do it. The
@@ -135,6 +193,7 @@
 #' then be automatically generated.
 #' 
 #' @section YAML:
+#' 
 #' A **pointblank** agent can be written to YAML with [yaml_write()] and the
 #' resulting YAML can be used to regenerate an agent (with [yaml_read_agent()])
 #' or interrogate the target table (via [yaml_agent_interrogate()]). When
@@ -149,7 +208,7 @@
 #' ```r
 #' agent %>% 
 #'   col_vals_decreasing(
-#'     columns = vars(a),
+#'     columns = a,
 #'     allow_stationary = TRUE,
 #'     increasing_tol = 0.5,
 #'     na_pass = TRUE,
@@ -166,7 +225,7 @@
 #' ```yaml
 #' steps:
 #' - col_vals_decreasing:
-#'     columns: vars(a)
+#'     columns: c(a)
 #'     allow_stationary: true
 #'     increasing_tol: 0.5
 #'     na_pass: true
@@ -186,26 +245,6 @@
 #' to preview the transformation of an agent to YAML without any writing to disk
 #' by using the [yaml_agent_string()] function.
 #' 
-#' @inheritParams col_vals_gt
-#' @param allow_stationary An option to allow pauses in decreasing values. For
-#'   example if the values for the test units are `[85, 82, 82, 80, 77]` then
-#'   the third unit (`82`, appearing a second time) would be marked with *fail*
-#'   when `allow_stationary` is `FALSE` (the default). Using `allow_stationary =
-#'   TRUE` will result in all the test units in `[85, 82, 82, 80, 77]` to be
-#'   marked with *pass*.
-#' @param increasing_tol An optional threshold value that allows for movement of
-#'   numerical values in the positive direction. By default this is `NULL` but
-#'   using a numerical value with set the absolute threshold of positive travel
-#'   allowed across numerical test units. Note that setting a value here also
-#'   has the effect of setting `allow_stationary` to `TRUE`.
-#' 
-#' @return For the validation function, the return value is either a
-#'   `ptblank_agent` object or a table object (depending on whether an agent
-#'   object or a table was passed to `x`). The expectation function invisibly
-#'   returns its input but, in the context of testing data, the function is
-#'   called primarily for its potential side-effects (e.g., signaling failure).
-#'   The test function returns a logical value.
-#' 
 #' @section Examples:
 #' 
 #' The `game_revenue` dataset in the package has the column `session_start`,
@@ -239,7 +278,7 @@
 #' agent <-
 #'   create_agent(tbl = game_revenue_2) %>%
 #'   col_vals_decreasing(
-#'     columns = vars(time_left),
+#'     columns = time_left,
 #'     allow_stationary = TRUE
 #'   ) %>%
 #'   interrogate()
@@ -264,7 +303,7 @@
 #' ```{r}
 #' game_revenue_2 %>%
 #'   col_vals_decreasing(
-#'     columns = vars(time_left),
+#'     columns = time_left,
 #'     allow_stationary = TRUE
 #'   ) %>%
 #'   dplyr::select(time_left) %>%
@@ -280,7 +319,7 @@
 #' ```r
 #' expect_col_vals_decreasing(
 #'   game_revenue_2,
-#'   columns = vars(time_left),
+#'   columns = time_left,
 #'   allow_stationary = TRUE
 #' )
 #' ```
@@ -293,7 +332,7 @@
 #' ```{r}
 #' game_revenue_2 %>%
 #'   test_col_vals_decreasing(
-#'     columns = vars(time_left),
+#'     columns = time_left,
 #'     allow_stationary = TRUE
 #'   )
 #' ```
@@ -326,13 +365,10 @@ col_vals_decreasing <- function(
     active = TRUE
 ) {
   
-  # Get `columns` as a label
-  columns_expr <- 
-    rlang::as_label(rlang::quo(!!enquo(columns))) %>%
-    gsub("^\"|\"$", "", .)
-  
   # Capture the `columns` expression
   columns <- rlang::enquo(columns)
+  # Get `columns` as a label
+  columns_expr <- as_columns_expr(columns)
   
   # Resolve the columns based on the expression
   columns <- resolve_columns(x = x, var_expr = columns, preconditions)
@@ -359,7 +395,7 @@ col_vals_decreasing <- function(
     secret_agent <-
       create_agent(x, label = "::QUIET::") %>%
       col_vals_decreasing(
-        columns = columns,
+        columns = tidyselect::all_of(columns),
         allow_stationary = allow_stationary,
         increasing_tol = increasing_tol,
         na_pass = na_pass,
@@ -401,6 +437,7 @@ col_vals_decreasing <- function(
   
   # Add one or more validation steps based on the
   # length of the `columns` variable
+  label <- resolve_label(label, columns, segments_list)
   for (i in seq_along(columns)) {
     for (j in seq_along(segments_list)) {
       
@@ -422,7 +459,7 @@ col_vals_decreasing <- function(
           seg_val = seg_val,
           actions = covert_actions(actions, agent),
           step_id = step_id[i],
-          label = label,
+          label = label[[i, j]],
           brief = brief[i],
           active = active
         )
