@@ -1,24 +1,28 @@
-#
-#                _         _    _      _                _    
-#               (_)       | |  | |    | |              | |   
-#  _ __    ___   _  _ __  | |_ | |__  | |  __ _  _ __  | | __
-# | '_ \  / _ \ | || '_ \ | __|| '_ \ | | / _` || '_ \ | |/ /
-# | |_) || (_) || || | | || |_ | |_) || || (_| || | | ||   < 
-# | .__/  \___/ |_||_| |_| \__||_.__/ |_| \__,_||_| |_||_|\_\
-# | |                                                        
-# |_|                                                        
+#------------------------------------------------------------------------------#
 # 
-# This file is part of the 'rich-iannone/pointblank' package.
+#                 _         _    _      _                _    
+#                (_)       | |  | |    | |              | |   
+#   _ __    ___   _  _ __  | |_ | |__  | |  __ _  _ __  | | __
+#  | '_ \  / _ \ | || '_ \ | __|| '_ \ | | / _` || '_ \ | |/ /
+#  | |_) || (_) || || | | || |_ | |_) || || (_| || | | ||   < 
+#  | .__/  \___/ |_||_| |_| \__||_.__/ |_| \__,_||_| |_||_|\_\
+#  | |                                                        
+#  |_|                                                        
+#  
+#  This file is part of the 'rstudio/pointblank' project.
+#  
+#  Copyright (c) 2017-2024 pointblank authors
+#  
+#  For full copyright and license information, please look at
+#  https://rstudio.github.io/pointblank/LICENSE.html
 # 
-# (c) Richard Iannone <riannone@me.com>
-# 
-# For full copyright and license information, please look at
-# https://rich-iannone.github.io/pointblank/LICENSE.html
-#
+#------------------------------------------------------------------------------#
+
 
 #' Run several tests and a final validation in a serial manner
 #'
-#' @description 
+#' @description
+#' 
 #' The `serially()` validation function allows for a series of tests to run in
 #' sequence before either culminating in a final validation step or simply
 #' exiting the series. This construction allows for pre-testing that may make
@@ -48,9 +52,9 @@
 #' Here's an example of how to arrange expressions:
 #' 
 #' ```
-#' ~ test_col_exists(., columns = vars(count)),
-#' ~ test_col_is_numeric(., columns = vars(count)),
-#' ~ col_vals_gt(., columns = vars(count), value = 2)
+#' ~ test_col_exists(., columns = count),
+#' ~ test_col_is_numeric(., columns = count),
+#' ~ col_vals_gt(., columns = count, value = 2)
 #' ```
 #' 
 #' This series concentrates on the column called `count` and first checks
@@ -65,7 +69,37 @@
 #' decimal value between `0` and `1` (serving as a fractional threshold of
 #' failing test units).
 #' 
+#' @inheritParams col_vals_gt
+#' 
+#' @param ... *Test/validation expressions*
+#' 
+#'   `<test/validation expressions>` // **required** (or, use `.list`)
+#' 
+#'   A collection one-sided formulas that consist of `test_*()` function calls
+#'   (e.g., [test_col_vals_between()], etc.) arranged in sequence of intended
+#'   interrogation order. Typically, validations up until the final one would
+#'   have some `threshold` value set (default is `1`) for short circuiting
+#'   within the series. A finishing validation function call (e.g.,
+#'   [col_vals_increasing()], etc.) can optionally be inserted at the end of the
+#'   series, serving as a validation step that only undergoes interrogation if
+#'   the prior tests adequately pass. An example of this is
+#'   `~ test_column_exists(., a), ~ col_vals_not_null(., a)`).
+#'   
+#' @param .list *Alternative to `...`*
+#' 
+#'   `<list of multiple expressions>` // **required** (or, use `...`)
+#' 
+#'   Allows for the use of a list as an input alternative to `...`.
+#' 
+#' @return For the validation function, the return value is either a
+#'   `ptblank_agent` object or a table object (depending on whether an agent
+#'   object or a table was passed to `x`). The expectation function invisibly
+#'   returns its input but, in the context of testing data, the function is
+#'   called primarily for its potential side-effects (e.g., signaling failure).
+#'   The test function returns a logical value.
+#' 
 #' @section Supported Input Tables:
+#' 
 #' The types of data tables that are officially supported are:
 #' 
 #'  - data frames (`data.frame`) and tibbles (`tbl_df`)
@@ -83,14 +117,21 @@
 #' **pointblank**).
 #'
 #' @section Column Names:
-#' If providing multiple column names in any of the supplied validation steps,
-#' the result will be an expansion of sub-validation steps to that number of
-#' column names. Aside from column names in quotes and in `vars()`,
-#' **tidyselect** helper functions are available for specifying columns. They
-#' are: `starts_with()`, `ends_with()`, `contains()`, `matches()`, and
-#' `everything()`.
+#' 
+#' `columns` may be a single column (as symbol `a` or string `"a"`) or a vector
+#' of columns (`c(a, b, c)` or `c("a", "b", "c")`). `{tidyselect}` helpers
+#' are also supported, such as `contains("date")` and `where(is.double)`. If
+#' passing an *external vector* of columns, it should be wrapped in `all_of()`.
+#' 
+#' When multiple columns are selected by `columns`, the result will be an
+#' expansion of validation steps to that number of columns (e.g.,
+#' `c(col_a, col_b)` will result in the entry of two validation steps).
+#' 
+#' Previously, columns could be specified in `vars()`. This continues to work, 
+#' but `c()` offers the same capability and supersedes `vars()` in `columns`.
 #' 
 #' @section Preconditions:
+#' 
 #' Providing expressions as `preconditions` means **pointblank** will preprocess
 #' the target table during interrogation as a preparatory step. It might happen
 #' that a particular validation requires a calculated column, some filtering of
@@ -109,6 +150,7 @@
 #' be supplied (e.g., `function(x) dplyr::mutate(x, col_b = col_a + 10)`).
 #' 
 #' @section Actions:
+#' 
 #' Often, we will want to specify `actions` for the validation. This argument,
 #' present in every validation function, takes a specially-crafted list
 #' object that is best produced by the [action_levels()] function. Read that
@@ -123,7 +165,19 @@
 #' quarter of the total test units fails, the other `stop()`s at the same
 #' threshold level).
 #' 
+#' @section Labels:
+#' 
+#' `label` may be a single string or a character vector that matches the number
+#' of expanded steps. `label` also supports `{glue}` syntax and exposes the
+#' following dynamic variables contextualized to the current step:
+#'   
+#' - `"{.step}"`: The validation step name
+#'     
+#' The glue context also supports ordinary expressions for further flexibility
+#' (e.g., `"{toupper(.step)}"`) as long as they return a length-1 string.
+#' 
 #' @section Briefs:
+#' 
 #' Want to describe this validation step in some detail? Keep in mind that this
 #' is only useful if `x` is an *agent*. If that's the case, `brief` the agent
 #' with some text that fits. Don't worry if you don't want to do it. The
@@ -131,6 +185,7 @@
 #' then be automatically generated.
 #' 
 #' @section YAML:
+#' 
 #' A **pointblank** agent can be written to YAML with [yaml_write()] and the
 #' resulting YAML can be used to regenerate an agent (with [yaml_read_agent()])
 #' or interrogate the target table (via [yaml_agent_interrogate()]). When
@@ -145,9 +200,9 @@
 #' ```r
 #' agent %>% 
 #'   serially(
-#'     ~ col_vals_lt(., columns = vars(a), value = 8),
-#'     ~ col_vals_gt(., columns = vars(c), value = vars(a)),
-#'     ~ col_vals_not_null(., columns = vars(b)),
+#'     ~ test_col_vals_lt(., columns = a, value = 8),
+#'     ~ test_col_vals_gt(., columns = c, value = vars(a)),
+#'     ~ col_vals_not_null(., columns = b),
 #'     preconditions = ~ . %>% dplyr::filter(a < 10),
 #'     actions = action_levels(warn_at = 0.1, stop_at = 0.2), 
 #'     label = "The `serially()` step.",
@@ -161,9 +216,9 @@
 #' steps:
 #' - serially:
 #'     fns:
-#'     - ~col_vals_lt(., columns = vars(a), value = 8)
-#'     - ~col_vals_gt(., columns = vars(c), value = vars(a))
-#'     - ~col_vals_not_null(., vars(b))
+#'     - ~test_col_vals_lt(., columns = a, value = 8)
+#'     - ~test_col_vals_gt(., columns = c, value = vars(a))
+#'     - ~col_vals_not_null(., columns = b)
 #'     preconditions: ~. %>% dplyr::filter(a < 10)
 #'     actions:
 #'       warn_fraction: 0.1
@@ -179,25 +234,6 @@
 #' possible to preview the transformation of an agent to YAML without any
 #' writing to disk by using the [yaml_agent_string()] function.
 #'
-#' @inheritParams col_vals_gt
-#' @param ... A collection one-sided formulas that consist of `test_*()`
-#'   function calls (e.g., [test_col_vals_between()], etc.) arranged in sequence
-#'   of intended interrogation order. Typically, validations up until the final
-#'   one would have some `threshold` value set (default is `1`) for short
-#'   circuiting within the series. A finishing validation function call (e.g.,
-#'   [col_vals_increasing()], etc.) can optionally be inserted at the end of the
-#'   series, serving as a validation step that only undergoes interrogation if
-#'   the prior tests adequately pass. An example of this is
-#'   `~ test_column_exists(., vars(a)), ~ col_vals_not_null(., vars(a))`).
-#' @param .list Allows for the use of a list as an input alternative to `...`.
-#' 
-#' @return For the validation function, the return value is either a
-#'   `ptblank_agent` object or a table object (depending on whether an agent
-#'   object or a table was passed to `x`). The expectation function invisibly
-#'   returns its input but, in the context of testing data, the function is
-#'   called primarily for its potential side-effects (e.g., signaling failure).
-#'   The test function returns a logical value.
-#'
 #' @section Examples:
 #' 
 #' For all examples here, we'll use a simple table with three numeric columns
@@ -229,9 +265,9 @@
 #' agent_1 <-
 #'   create_agent(tbl = tbl) %>%
 #'   serially(
-#'     ~ test_col_is_numeric(., columns = vars(a, b)),
-#'     ~ test_col_vals_not_null(., columns = vars(a, b)),
-#'     ~ col_vals_gt(., columns = vars(b), value = vars(a))
+#'     ~ test_col_is_numeric(., columns = c(a, b)),
+#'     ~ test_col_vals_not_null(., columns = c(a, b)),
+#'     ~ col_vals_gt(., columns = b, value = vars(a))
 #'     ) %>%
 #'   interrogate()
 #' ```
@@ -256,8 +292,8 @@
 #' agent_2 <-
 #'   create_agent(tbl = tbl) %>%
 #'   serially(
-#'     ~ test_col_is_numeric(., columns = vars(a, b)),
-#'     ~ test_col_vals_not_null(., columns = vars(a, b))
+#'     ~ test_col_is_numeric(., columns = c(a, b)),
+#'     ~ test_col_vals_not_null(., columns = c(a, b))
 #'   ) %>%
 #'   interrogate()
 #' ```
@@ -279,9 +315,9 @@
 #' ```{r}
 #' tbl %>%
 #'   serially(
-#'     ~ test_col_is_numeric(., columns = vars(a, b)),
-#'     ~ test_col_vals_not_null(., columns = vars(a, b)),
-#'     ~ col_vals_gt(., columns = vars(b), value = vars(a))
+#'     ~ test_col_is_numeric(., columns = c(a, b)),
+#'     ~ test_col_vals_not_null(., columns = c(a, b)),
+#'     ~ col_vals_gt(., columns = b, value = vars(a))
 #'   )
 #' ```
 #'
@@ -293,9 +329,9 @@
 #' ```r
 #' expect_serially(
 #'   tbl,
-#'   ~ test_col_is_numeric(., columns = vars(a, b)),
-#'   ~ test_col_vals_not_null(., columns = vars(a, b)),
-#'   ~ col_vals_gt(., columns = vars(b), value = vars(a))
+#'   ~ test_col_is_numeric(., columns = c(a, b)),
+#'   ~ test_col_vals_not_null(., columns = c(a, b)),
+#'   ~ col_vals_gt(., columns = b, value = vars(a))
 #' )
 #' ```
 #' 
@@ -307,9 +343,9 @@
 #' ```{r}
 #' tbl %>%
 #'   test_serially(
-#'     ~ test_col_is_numeric(., columns = vars(a, b)),
-#'     ~ test_col_vals_not_null(., columns = vars(a, b)),
-#'     ~ col_vals_gt(., columns = vars(b), value = vars(a))
+#'     ~ test_col_is_numeric(., columns = c(a, b)),
+#'     ~ test_col_vals_not_null(., columns = c(a, b)),
+#'     ~ col_vals_gt(., columns = b, value = vars(a))
 #'   )
 #' ```
 #'
@@ -624,6 +660,7 @@ serially <- function(
   
   # Add one or more validation steps based on the
   # length of `segments_list`
+  label <- resolve_label(label, segments = segments_list)
   for (i in seq_along(segments_list)) {
     
     seg_col <- names(segments_list[i])
@@ -644,7 +681,7 @@ serially <- function(
         seg_val = seg_val,
         actions = covert_actions(actions, agent),
         step_id = step_id,
-        label = label,
+        label = label[[i]],
         brief = brief,
         active = active
       )
