@@ -285,13 +285,7 @@ file_tbl <- function(
     verify = TRUE
 ) {
   
-  if (!requireNamespace("readr", quietly = TRUE)) {
-    stop(
-      "Reading a table from a file requires the readr package:\n",
-      " * It can be installed with `install.packages(\"readr\")`.",
-      call. = FALSE
-    )
-  }
+  rlang::check_installed("readr", "to read a table from a file.")
   
   file_extension <- tolower(tools::file_ext(file))
   file_name <- basename(file)
@@ -529,13 +523,7 @@ from_github <- function(
     
   } else if (grepl("#", repo, fixed = TRUE)) {
     
-    if (!requireNamespace("jsonlite", quietly = TRUE)) {
-      stop(
-        "Getting a table from a file in a PR requires the jsonlite package:\n",
-        " * It can be installed with `install.packages(\"jsonlite\")`.",
-        call. = FALSE
-      )
-    }
+    rlang::check_installed("jsonlite", "to get a table from a file in a PR.")
 
     pr_number <- unlist(strsplit(repo, "#"))[2]
     pulls_doc_tempfile <- tempfile(pattern = "pulls", fileext = ".json")
