@@ -687,6 +687,7 @@ yaml_agent_string <- function(
 }
 
 as_c_fn <- function(columns) {
+  columns <- strsplit(unlist(columns), ", ")[[1]]
   paste0("c(", paste0('"', columns, '"', collapse = ", "), ")")
 }
 
@@ -703,9 +704,9 @@ as_list_preconditions <- function(preconditions) {
         gsub("function (x) \n{", "function(x) {", ., fixed = TRUE)
     )
     
-  } else {
-    return(as.character(preconditions))
   }
+  
+  as.character(preconditions)
 }
 
 as_list_segments <- function(segments) {
@@ -740,9 +741,9 @@ as_list_active <- function(active) {
 
   if (is.logical(active[[1]])) {
     return(active[[1]])
-  } else {
-    return(as.character(active))
   }
+  
+  as.character(active)
 }
 
 to_list_action_levels <- function(actions) {
