@@ -3,6 +3,8 @@ from __future__ import annotations
 import copy
 from dataclasses import dataclass
 
+import narwhals as nw
+
 from pointblank._constants import IBIS_BACKENDS
 from pointblank._utils import _get_tbl_type, _is_lazy_frame, _is_lib_present, _is_narwhals_table
 
@@ -59,10 +61,15 @@ class Schema:
 
     - Polars DataFrame (`"polars"`)
     - Pandas DataFrame (`"pandas"`)
+    - PySpark table (`"pyspark"`)
     - DuckDB table (`"duckdb"`)*
     - MySQL table (`"mysql"`)*
     - PostgreSQL table (`"postgresql"`)*
     - SQLite table (`"sqlite"`)*
+    - Microsoft SQL Server table (`"mssql"`)*
+    - Snowflake table (`"snowflake"`)*
+    - Databricks table (`"databricks"`)*
+    - BigQuery table (`"bigquery"`)*
     - Parquet table (`"parquet"`)*
 
     The table types marked with an asterisk need to be prepared as Ibis tables (with type of
@@ -336,6 +343,16 @@ class Schema:
             schema_dict = {k: str(v) for k, v in schema_dict.items()}
             self.columns = list(schema_dict.items())
 
+        elif table_type == "pyspark":
+            # Convert PySpark DataFrame to Narwhals to get schema
+            nw_df = nw.from_native(self.tbl)
+            if _is_lazy_frame(data=nw_df):
+                schema_dict = dict(nw_df.collect_schema())
+            else:
+                schema_dict = dict(nw_df.schema.items())
+            schema_dict = {k: str(v) for k, v in schema_dict.items()}
+            self.columns = list(schema_dict.items())
+
         elif table_type in IBIS_BACKENDS:
             schema_dict = dict(self.tbl.schema().items())
             schema_dict = {k: str(v) for k, v in schema_dict.items()}
